/*  sweopts.h
    Bryan Guillaume & Tom Nichols
    Copyright (C) 2019 University of Oxford  */

#if !defined(sweopts_h)
#define sweopts_h

#include <string>
#include <stdlib.h>
#include <stdio.h>
#include "utils/options.h"
#include "utils/log.h"

namespace SWE {

	class sweopts {
	 public:
		static sweopts& getInstance();
		~sweopts() { delete gopt; }

		Utilities::Option<std::string> in_fileroot;
		Utilities::Option<std::string> maskname;
		Utilities::Option<std::string> out_fileroot;
		Utilities::Option<std::string> dm_file;
		Utilities::Option<std::string> tc_file;
		Utilities::Option<std::string> fc_file;
		Utilities::Option<std::string> subj_file;
		Utilities::Option<std::string> resamplingMatrix_file;
		Utilities::Option<bool> modifiedSwE;
		Utilities::Option<bool> wb;
		Utilities::Option<bool> logP;
		Utilities::Option<int> n_boot;
		Utilities::Option<bool> voxelwiseOutput;
		Utilities::Option<bool> doFOnly;
		Utilities::Option<bool> tfce;
		Utilities::Option<bool> tfce2D;
		Utilities::Option<float> cluster_thresh;
		Utilities::Option<float> clustermass_thresh;
		Utilities::Option<float> f_thresh;
		Utilities::Option<float> fmass_thresh;
		Utilities::Option<float> tfce_height;
		Utilities::Option<float> tfce_delta;
		Utilities::Option<float> tfce_size;
		Utilities::Option<int> tfce_connectivity;
		Utilities::Option<bool> help;
		Utilities::Option<bool> verbose;
		Utilities::Option<bool> outputRaw;
		Utilities::Option<bool> outputEquivalent;
		Utilities::Option<bool> outputDof;
		Utilities::Option<bool> outputUncorr;
//		Utilities::Option<bool> outputTextWB;
		Utilities::Option<bool> outputTextNull;
		Utilities::Option<bool> disableNonConstantMask;
		Utilities::Option<int> randomSeed;
		Utilities::Option<std::vector<int> > voxelwise_ev_numbers;
		Utilities::Option<std::vector<std::string> > voxelwise_ev_filenames;
		Utilities::Option<bool> outputGlm;

      void parse_command_line(int argc, char** argv, Utilities::Log& logger);

	 private:
		sweopts();
		const sweopts& operator=(sweopts&);
		sweopts(sweopts&);

		Utilities::OptionParser options;

		static sweopts* gopt;
	};

 inline sweopts& sweopts::getInstance(){
	 if(gopt == NULL)
		 gopt = new sweopts();

	 return *gopt;
 }

 inline sweopts::sweopts() :
	in_fileroot(std::string("-i"), "",
							std::string("~<input>\t4D input image"),
							true, Utilities::requires_argument),
	maskname(std::string("-m,--mask"), "",
					 std::string("~<mask>\tmask image"),
					 false, Utilities::requires_argument),
	out_fileroot(std::string("-o"), std::string(""),
							 std::string("~<out_root>\toutput file-rootname"),
							 true, Utilities::requires_argument),
	dm_file(std::string("-d"), std::string(""),
					std::string("~<design.mat>\tdesign matrix file"),
					true, Utilities::requires_argument),
	tc_file(std::string("-t"), std::string(""),
					std::string("~<design.con>\tt contrasts file"),
					true, Utilities::requires_argument),
	fc_file(std::string("-f"), std::string(""),
					std::string("~<design.fts>\tf contrasts file"),
					false, Utilities::requires_argument),
	subj_file(std::string("-s,--subj"), std::string(""),
						std::string("~<design.sub>\tsubjects file"),
						true, Utilities::requires_argument),
	resamplingMatrix_file(std::string("--resamplingMatrix"), std::string(""),
												std::string("~<resamplingMatrix>\tresampling matrix file"),
												false, Utilities::requires_argument, false),
	modifiedSwE(std::string("--modified"), false,
							std::string("use the modified \"Homogeneous\" SwE instead of the classic \"Heterogeneous\" SwE"),
							false, Utilities::no_argument),
	wb(std::string("--wb"), false,
							std::string("\tinference is done using a non-parametric Wild Bootstrap procedure instead of a parametric procedure"),
							false, Utilities::no_argument),
	logP(std::string("--logp"), false,
		 std::string("\treturn -log_10(p) images instead of 1-p images"),
		 false, Utilities::no_argument),
	n_boot(std::string("-n"), 999,
				std::string("~<n_boot>\tnumber of bootstraps (default 999)"),
				false, Utilities::requires_argument),
	voxelwiseOutput(std::string("-x,--corrp"),false,
									std::string("output voxelwise corrected p-value images"),
									false, Utilities::no_argument),
	doFOnly(std::string("--fonly"), false,
					std::string("\tcalculate f-statistics only"),
					false, Utilities::no_argument),
	tfce(std::string("-T"), false,
			std::string("\tcarry out Threshold-Free Cluster Enhancement"),
			false, Utilities::no_argument),
	tfce2D(std::string("--T2"), false,
				std::string("\tcarry out Threshold-Free Cluster Enhancement with 2D optimisation (e.g. for TBSS data); H=2, E=1, C=26"),
				false, Utilities::no_argument),
	cluster_thresh(std::string("-c"), -1,
								 std::string("~<thresh>\tcarry out cluster-extent-based inference for t-contrasts with the supplied cluster-forming threshold (supplied as an equivalent z-score if thresh >= 1 or as an uncorrected p-value if thresh < 1)"),
								 false, Utilities::requires_argument),
	clustermass_thresh(std::string("-C"), -1,
								 std::string("~<thresh>\tcarry out cluster-mass-based inference for t-contrasts with the supplied cluster-forming threshold (supplied as an equivalent z-score if thresh >= 1 or as an uncorrected p-value if thresh < 1)"),
									false, Utilities::requires_argument),
	f_thresh(std::string("-F"), -1,
					 std::string("~<thresh>\tcarry out cluster-extent-based inference for f-contrasts with the supplied cluster-forming threshold (supplied as an equivalent one-degree-of-freedom chi-squared-score if thresh >= 1 or as an uncorrected p-value if thresh < 1)"),
					 false, Utilities::requires_argument),
	fmass_thresh(std::string("-S"), -1,
							 std::string("~<thresh>\tcarry out cluster-mass-based inference for f-contrasts with the supplied cluster-forming threshold (supplied as an equivalent one-degree-of-freedom chi-squared-score if thresh >= 1 or as an uncorrected p-value if thresh < 1)"),
							 false, Utilities::requires_argument),
	tfce_height(std::string("--tfce_H"), 2, std::string("~<H>\tTFCE height parameter (default=2)"), false, Utilities::requires_argument),
	tfce_delta(std::string("--tfce_D"), 1, std::string("~<H>\tTFCE delta parameter overide"), false, Utilities::requires_argument),
	tfce_size(std::string("--tfce_E"), 0.5, std::string("~<E>\tTFCE extent parameter (default=0.5)"), false, Utilities::requires_argument),
	tfce_connectivity(std::string("--tfce_C"), 6, std::string("~<C>\tTFCE connectivity (6 or 26; default=6)"), false, Utilities::requires_argument),
	help(std::string("-h,--help"), false,
			 std::string("display this message"),
			 false, Utilities::no_argument),
	verbose(std::string("--quiet"), true,
					std::string("\tswitch off diagnostic messages"),
					false, Utilities::no_argument),
	outputRaw(std::string("-R,--raw"), false,
						std::string("output raw voxelwise statistic images"),
						false, Utilities::no_argument),
	outputEquivalent(std::string("-E,--equivalent"), false,
						std::string("output equivalent z (for t-contrast) or one-degree-of-freedom chi-squared (for f-contrast) statistic images"),
						false, Utilities::no_argument),
	outputDof(std::string("-D,--dof"), false,
									 std::string("output effective number of degrees of freedom images"),
									 false, Utilities::no_argument),
	outputUncorr(std::string("--uncorrp"), false,
							 std::string("output uncorrected p-value images"),
							 false, Utilities::no_argument),
//	outputTextWB(std::string("-P"), false,
//								 std::string("\toutput Wild Boostrap vector text file"),
//								 false, Utilities::no_argument),
	outputTextNull(std::string("-N"), false,
								 std::string("\toutput null distribution text files"),
								 false, Utilities::no_argument),
	disableNonConstantMask(std::string("--norcmask"), false,
												 std::string("don't remove constant voxels from mask"),
												 false, Utilities::no_argument),
	randomSeed(std::string("--seed"),0,
						 std::string("~<seed>\tspecific integer seed for random number generator"),
						 false, Utilities::requires_argument),
	voxelwise_ev_numbers(std::string("--vxl"), std::vector<int>(),
											 std::string("\tlist of numbers indicating voxelwise EVs position in the design matrix (list order corresponds to files in vxf option)."),
											 false, Utilities::requires_argument),
	voxelwise_ev_filenames(std::string("--vxf"), std::vector<std::string>(),
												 std::string("\tlist of 4D images containing voxelwise EVs (list order corresponds to numbers in vxl option)."),
												 false, Utilities::requires_argument),
	outputGlm(std::string("--glm_output"), false,
						std::string("output glm information (pe, cope & varcope)"),
						false, Utilities::no_argument),
	options("swe v1.0.3", "swe -i <input> -o <output> -d <design.mat> -t <design.con> -s <design.sub> [options]")
	{

		try {
//			options.add(demean_data);
			options.add(in_fileroot);
			options.add(maskname);
			options.add(out_fileroot);
			options.add(dm_file);
			options.add(tc_file);
			options.add(fc_file);
			options.add(subj_file);
			options.add(resamplingMatrix_file);
			options.add(modifiedSwE);
			options.add(wb);
			options.add(logP);
			options.add(n_boot);
			options.add(voxelwiseOutput);
			options.add(doFOnly);
			options.add(tfce);
			options.add(tfce2D);
			options.add(cluster_thresh);
			options.add(clustermass_thresh);
			options.add(f_thresh);
			options.add(fmass_thresh);
			options.add(help);
			options.add(verbose);
			options.add(outputRaw);
			options.add(outputEquivalent);
			options.add(outputDof);
			options.add(outputUncorr);
//			options.add(outputTextWB);
			options.add(outputTextNull);
			options.add(disableNonConstantMask);
			options.add(randomSeed);
			options.add(tfce_height);
			options.add(tfce_delta);
			options.add(tfce_size);
			options.add(tfce_connectivity);
			options.add(voxelwise_ev_numbers);
			options.add(voxelwise_ev_filenames);
			options.add(outputGlm);
		}
		catch(Utilities::X_OptionError& e) {
			options.usage();
            std::cerr << std::endl << e.what() << std::endl;
		}
		catch(std::exception &e) {
			std::cerr << e.what() << std::endl;
		}
	}
}

#endif
